/**
 *  Copyright 2009-2010 Matus Chochlik. Distributed under the Boost
 *  Software License, Version 1.0. (See accompanying file
 *  LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 *
 *  WARNING: this file is here for development / testing purposes only
 *  and its contents and input / output can change witout any prior
 *  notice.
 */

#include <mirror/mirror.hpp>
#include <mirror/stream.hpp>
#include <mirror/utils/quick_reg.hpp>
#include <iostream>
//

namespace test {

class foo
{
public:
	int a;
	int b;
	char c;
};

struct bar
{
	double d, e;
	float f;
};

class foobar : public foo, public bar
{
private:
	MIRROR_FRIENDLY_CLASS(foobar);
	short g,h;
public:
	foobar(void)
	 : g(0)
	 , h(h)
	{
		a = b = 0;
		c = '\0';
		d = e = 0.0;
		f = 0.0f;
	}
};

} // namespace test

MIRROR_NAMESPACE_BEGIN

/*
*/


MIRROR_QREG_GLOBAL_SCOPE_NAMESPACE(test)

MIRROR_QREG_POD_CLASS(class,  test, foo, (a)(b)(c))
MIRROR_QREG_POD_CLASS(struct, test, bar, (d)(e)(f))

MIRROR_QREG_CLASS(class, test, foobar, (
	(test::foo)(test::bar),
	(g)(h)
))

MIRROR_NAMESPACE_END

int main(void)
{
	try
	{
		using namespace mirror;
		//
		static_assert(
			mp::size<member_variables<
				MIRRORED_CLASS(test::foo)
			> >::value == 3, "test::foo reg error"
		);
		static_assert(
			mp::size<member_variables<
				MIRRORED_CLASS(test::bar)
			> >::value == 3, "test::bar reg error"
		);
		static_assert(
			mp::size<base_classes<
				MIRRORED_CLASS(test::foobar)
			> >::value == 2, "test::foobar reg error"
		);
	}
	catch(const std::exception& error)
	{
		std::cerr << "Error: " << error.what() << std::endl;
	}
	//
	return 0;
}

