/**
 *  Copyright 2009-2011 Matus Chochlik. Distributed under the Boost
 *  Software License, Version 1.0. (See accompanying file
 *  LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 *
 *  WARNING: this file is here for development / testing purposes only
 *  and its contents and input / output can change witout any prior
 *  notice.
 */

#include <mirror/all.hpp>
#include <mirror/stream.hpp>
#include <iostream>
#include <cassert>

MIRROR_NAMESPACE_BEGIN

namespace aux {

} // namespace aux

MIRROR_NAMESPACE_END

#include "./test.hpp"
int main(void)
{
	//
	using namespace mirror;
	typedef std::pair<int, char> E;
	std::vector<E> x = {
		{9, 'a'},
		{8, 'b'},
		{7, 'c'},
		{6, 'd'},
		{5, 'e'},
		{4, 'f'},
		{3, 'g'},
		{2, 'h'},
		{1, 'i'},
		{0, 'j'}
	};
	typedef reflected<decltype(x)>::type meta_xt;
	typedef mp::front<all_containers<meta_xt> >::type meta_cntnr_xt;
	typedef default_traversal<meta_cntnr_xt>::type meta_trav_xt;
	//
	static_assert(mirror::mp::is_a<meta_xt, mirror::meta_type_tag>::value, "1");
	static_assert(mirror::mp::is_a<meta_xt, mirror::meta_class_tag>::value, "2");
	static_assert(mirror::mp::is_a<meta_cntnr_xt, mirror::meta_container_tag>::value, "3");
	static_assert(mirror::mp::is_a<meta_trav_xt, mirror::meta_traversal_tag>::value, "4");
	//
	std::cout << meta_cntnr_xt::element_type::base_name() << std::endl;
	std::cout << meta_trav_xt::base_name() << std::endl;
	std::cout << meta_cntnr_xt::size_hint(x) << std::endl;
	//
	for(auto trav = meta_trav_xt::start(x); !trav.done(); trav.step_front())
		std::cout << trav.front().get().first << std::endl;
	//
	typedef mp::front<inserters<meta_cntnr_xt> >::type meta_insert;
	meta_insert::insert(x, {-1, 'x'});
	meta_insert::insert(x, {-2, 'y'});
	meta_insert::insert(x, {-3, 'z'});
	//
	for(auto trav = meta_trav_xt::start(x); !trav.done(); trav.step_front())
		std::cout << trav.front().get().first << std::endl;
	//
	std::cout << stream::to_json::from(
		x,[](std::ostream& out){ out << "x"; }
	) << std::endl;
	std::cout << stream::to_simple_struct::from(
		x,[](std::ostream& out){ out << "x"; }
	) << std::endl;
	//
	typedef mp::front<erasers<meta_cntnr_xt> >::type meta_erase;
	meta_erase::erase(x);
	//
	std::cout << meta_cntnr_xt::size_hint(x) << std::endl;
	//
	//
	int y[10] = {9,8,7,6,5,4,3,2,1,0};
	//
	typedef reflected<decltype(y)>::type meta_yt;
	typedef mp::front<containers<meta_yt> >::type meta_cntnr_yt;
	typedef default_traversal<meta_cntnr_yt>::type meta_trav_yt;
	//
	std::cout << meta_cntnr_yt::element_type::base_name() << std::endl;
	std::cout << meta_trav_yt::base_name() << std::endl;
	std::cout << meta_cntnr_yt::size_hint(y) << std::endl;
	//
	for(auto trav = meta_trav_yt::start(y); !trav.done(); trav.step_front())
		std::cout << trav.front().get() << std::endl;
	//
	return 0;
}

